#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Crée le Mon Feb 14 10:34:29 2022
@auteur: david ALBERTO
(www.astrolabe-science.fr)
Ce code utilise le module mendeleev (données sur les éléments chimiques)
Dans ce module, un élément est appelé par element(N).
"""

 # Paramètres personnalisables-----
"""
Les paramètres suivants sont liés entre eux, pour le rendu final.
Ils doivent être ajustés manuellement :
    - dimensions de la figure
    - valeurs des paramètres d'échelle a et b
    - taille des caractères
    - les différents offsets en x et en y
"""
Zmax=118
#facteurs d'échelle :
a=0.22 #en abscisses
b=0.28 #en ordonnées. NB : les ordonnées sont négatives

# Réglages d'écriture du symbole des éléments :
taille_symbole=19
c_symbole='k'#couleur symbole

# Réglages d'écriture du numéro atomique :
x_offset_Z=-0.32
y_offset_Z=0.33
taille_Z=9
c_Z='k'#couleur numéro atomique

# Réglages d'écriture des terres rares :
y_offset_La=2.5#décalage vertical lanthanides
y_offset_Ac=y_offset_La#décalage vertical actinides

# Réglages d'écriture du nom des éléments :
x_offset_nom=-0.0
y_offset_nom=-0.4
c_nom='k'
taille_nom=7


# ---------------------------------
# Appel des modules
import matplotlib.pyplot as plt
from mendeleev import element
import matplotlib as mpl#Normalisation des valeurs
import matplotlib.colorbar as cbar#barres de couleurs
from mpl_toolkits.axes_grid1 import make_axes_locatable#positionner la barre de couleurs

def N_isotopes(N):
    # renvoie le nombre d'isotopes de l'élément d'après son n° atomique
    return len(element(N).isotopes)

#----------------gamme de couleurs choisie (colormap) :--------------------------
"""
Les colormaps peuvent être choisis parmi les séquentiels :
'Greys', 'Purples', 'Blues', 'Greens', 'Oranges', 'Reds',
'YlOrBr', 'YlOrRd', 'OrRd', 'PuRd', 'RdPu', 'BuPu',
'GnBu', 'PuBu', 'YlGnBu', 'PuBuGn', 'BuGn', 'YlGn'
"""
cmap=plt.get_cmap('Blues',10) #ici, on demande 10 couleurs discontinues, car il y a max. 10 isotopes pour un élément.
#normalisation de la gamme de valeurs :
norm = mpl.colors.Normalize(vmin=1,vmax=11) 

#---------------------------------------------------------
noms=['Hydrogène','Hélium','Lithium','Béryllium','Bore','Carbone','Azote','Oxygène',
      'Fluor','Néon','Sodium','Magnésium','Aluminium','Silicium','Phosphore','Soufre',
      'Chlore','Argon','Potassium','Calcium','Scandium','Titane','Vanadium','Chrome',
      'Manganèse','Fer','Cobalt','Nickel','Cuivre','Zinc','Gallium','Germanium',
      'Arsenic','Sélénium','Brome','Krypton','Rubidium','Strontium','Yttrium',
      'Zirconium','Niobium','Molybdène','Technétium','Ruthénium','Rhodium',
      'Palladium','Argent','Cadmium','Indium',
'Étain','Antimoine','Tellure','Iode','Xénon','Césium','Baryum','Lanthane','Cérium',
'Praséodyme','Néodyme','Prométhium','Samarium','Europium','Gadolinium','Terbium',
'Dysprosium','Holmium','Erbium','Thulium','Ytterbium','Lutetium','Hafnium','Tantale',
'Tungstène','Rhénium','Osmium','Iridium','Platine','Or','Mercure','Thallium','Plomb',
'Bismuth','Polonium','Astate','Radon','Francium','Radium','Actinium','Thorium','Protactinium',
'Uranium','Neptunium','Plutonium','Americium','Curium','Berkelium','Californium','Einsteinium',
'Fermium','Mendelevium','Nobelium','Lawrencium','Rutherfordium','Dubnium','Seaborgium','Bohrium',
'Hassium','Meitnerium','Darmstadtium','Roentgenium','Copernicium','Nihonium','Flerovium',
'Moscovium','Livermorium','Tennessine','Oganesson',
]#le module mendeleev n'a que les noms des éléments en anglais
# ---------------------------------


#création des listes de coordonnées des éléments
"""
La ligne est la valeur de la période de l'élément.
La colonne est la valeur du paramètre "group_id" de l'élément.
Pour les terres rares, le module ne fournit pas de valeur pour "group_id" ; 
je me base donc sur la valeur de N pour établir un numéro de colonne.
Les listes "ligne" et "colonne" serviront ensuite pour toutes les indications
(N° atomique, symbole, nom).
"""

def coord_X(N):
    # renvoie l'abscisse de l'élément dans le graphique
    if N>=58 and N<=71:#lanthanides
        xx = (3+N-57)*a
    elif N>=90 and N<=103:#actinides
        xx=(3+N-89)*a
    else:
        xx=(element(N).group_id)*a
    return xx

def coord_Y(N):
    # renvoie l'ordonnée de l'élément dans le graphique
    if N>=58 and N<=71:#lanthanides
        yy=(element(N).period+y_offset_La)*b
    elif N>=90 and N<=103:#actinides
        yy=(element(N).period+y_offset_Ac)*b
    else:
        yy=(element(N).period)*b
    return yy

# ---------------------------------
# paramètres du graphique
fig=plt.figure(figsize=(29.7/2.54,21/2.54),tight_layout=True)

plt.rcParams["font.family"] = "Serif"
# plt.rcParams["font.family"] = "Liberation Serif"# ou autre police installée (police Serif conseillée)

plt.rcParams.update({"text.usetex": True}) #utiliser la police LaTeX (optionnel)
ax=plt.subplot(111)

plt.title(r"$\textsf{\textbf{Les éléments chimiques et leur nombre d'isotopes}}$",fontsize=22,y=0.92)
plt.xlim(0,19*a)
plt.ylim(-10.5*b,0)
plt.xticks([])
plt.yticks([])
ax.set_aspect('equal')
# ---------------------------------
# Remplissage du tableau :
for N in range(1,118+1):
    elem=element(N)
    plt.text(coord_X(N),-coord_Y(N),elem.symbol,fontsize=taille_symbole,va='center',ha='center')#symbole
    rectangle=plt.Rectangle((coord_X(N)-0.5*a,-(coord_Y(N)+0.5*b)),a,b,facecolor=cmap(norm(N_isotopes(N))),edgecolor='k')#case
    ax.add_patch(rectangle)
    if N in [91,101,104,110,116]:#éléments aux noms longs
        tailleDuNom=6
    else:tailleDuNom=taille_nom
    plt.text(coord_X(N)+x_offset_Z*a,-coord_Y(N)+y_offset_Z*b,N,c=c_Z,fontsize=taille_Z,ha='center')#numéro atomique
    plt.text(coord_X(N)+x_offset_nom*a,-coord_Y(N)+y_offset_nom*b,r'\textsf{%s}'%(noms[N-1]),c=c_nom,fontsize=tailleDuNom,ha='center')#nom
    # autre option, sans le code LaTeX :
    # plt.text(colonne[N-1]+x_offset_nom*a,-ligne[N-1]+y_offset_nom*b,noms[N-1],c=c_nom,fontsize=tailleDuNom,ha='center')#nom
    tailleDuNom=taille_nom


# Trait pour les terres rares :
plt.plot([3.5*a,3.5*a],[-7.5*b,-8.0*b],'--',c='gray',lw=1)

#tracé de la barre de couleurs :
divider = make_axes_locatable(ax)
cax = divider.append_axes('bottom', size='6%', pad=0.2)
cb=cbar.ColorbarBase(cax, cmap=cmap, norm=norm, orientation='horizontal')
cax.set_xlabel("Nombre d'isotopes", fontsize=12)

ax.text(0.985,0.3,r"D. $\textsc{Alberto}$ www.astrolabe-science.fr",
        rotation=90, fontsize=6, transform=ax.transAxes)

NomFichier="TableauPeriodique_isotopes"
fig.savefig(NomFichier+'.png',dpi=200)
fig.savefig(NomFichier+'.pdf')